#!/usr/bin/env python3
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__      = "Joel Boyd"
__copyright__   = "Copyright 2017"
__credits__     = ["Joel Boyd"]
__license__     = "GPL3"
__version__     = "0.0.7"
__maintainer__  = "Joel Boyd"
__email__       = "joel.boyd near uq.net.au"
__status__      = "Development"

################################################################################
import logging
import os
import subprocess
from enrichm.databases import Databases
from enrichm.data import Data
################################################################################
class Plot:

	def __init__(self):
		path_to_scripts = os.path.split(os.path.realpath(__file__))[0]
		self.draw_pca_script_path 		= os.path.join(path_to_scripts, "PLOT_ko_pca.r") 
		self.draw_heatmap_script_path 	= os.path.join(path_to_scripts, "PLOT_ko_heatmap.r") 
		self.draw_barplots_script_path 	= os.path.join(path_to_scripts, "PLOT_ko_breakdown.r") 
		self.ko00000 					= os.path.join(Data.DATABASE_DIR, Databases.DB_VERSION, 'ko00000.tsv')
		self.output_pca_plot = 'presence_absence_pca_plot.svg'
		self.output_heatmap_plot = 'presence_absence_pca_plot.svg'

	def draw_pca_plot(self, annotation_matrix, metadata, output_directory):
		logging.info('	- Generating PCA plot')
		output_path = os.path.join(output_directory, self.output_pca_plot)
		
		cmd = "Rscript %s -i %s -m %s -o %s > /dev/null 2>&1" \
			% (self.draw_pca_script_path, annotation_matrix, metadata, output_path)
		subprocess.call(cmd, shell=True)

	def draw_barplots(self, annotation_matrix, pvalue, output_directory):
		logging.info('	- Generating KO breakdown plots')
		cmd = "Rscript %s -i %s -o %s -k %s -p %f > /dev/null 2>&1" \
			% (self.draw_barplots_script_path, annotation_matrix, output_directory, self.ko00000, pvalue)
		subprocess.call(cmd, shell=True)



#!/usr/bin/env python3
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__      = "Joel Boyd"
__copyright__   = "Copyright 2017"
__credits__     = ["Joel Boyd"]
__license__     = "GPL3"
__version__     = "0.0.7"
__maintainer__  = "Joel Boyd"
__email__       = "joel.boyd near uq.net.au"
__status__      = "Development"

################################################################################
# Imports

from enrichm.databases import Databases

################################################################################
class Parser:
	
	@staticmethod
	def parse_genome_and_annotation_file_lf(self, genome_and_annotation_file):
		genome_to_annotation_sets = dict()
		
		for line in open(genome_and_annotation_file):
			try:
				genome, annotation = line.strip().split("\t")
			except:
				raise Exception("Input genomes/annotation file error on %s" % line)
			
			if genome not in genome_to_annotation_sets:
				genome_to_annotation_sets[genome] = set()

			genome_to_annotation_sets[genome].add(annotation)
		
		return genome_to_annotation_sets
	
	@staticmethod
	def parse_genome_and_annotation_file_matrix(self, genome_and_annotation_matrix):
		genome_and_annotation_matrix_io = open(genome_and_annotation_matrix)
		headers=genome_and_annotation_matrix_io.readline().strip().split('\t')[1:]
		genome_to_annotation_sets = {genome_name:set() for genome_name in headers}

		for line in genome_and_annotation_matrix_io:
			sline = line.strip().split('\t')
			annotation, entries = sline[0], sline[1:]
			for genome_name, entry in zip(headers, entries):
				if float(entry) > 0:
					genome_to_annotation_sets[genome_name].add(annotation)

		return genome_to_annotation_sets
	
	@staticmethod
	def parse_taxonomy(self, taxonomy_path):
		
		output_taxonomy_dictionary = dict()

		for line in open(taxonomy_path):
			genome, taxonomy_string = line.strip().split('\t')
			output_taxonomy_dictionary[genome] = taxonomy_string.split(';')
			
		return output_taxonomy_dictionary
	
	@staticmethod
	def _parse_simple_matrix(matrix, numeric = False):
		matrix_io = open(matrix)
		header_values = matrix_io.readline().strip().split('\t')[1:]
		output_dict = {header:{} for header in header_values}
		for line in matrix_io:
			sline = line.strip().split('\t')
			_, content = sline[0], sline[1:]

			for key, value in zip(header_values, content):

				if numeric == True:
					value = float(value)
				output_dict[key] = value

		return output_dict

	@staticmethod
	def _parse_matrix(self, matrix_file_io, colnames):
		for line in matrix_file_io:
			sline = line.strip().split('\t')
			rowname, entries = sline[0], sline[1:]
			for colname, entry in zip(colnames, entries):
				yield colname, entry, rowname
	
	@staticmethod
	def parse_tpm_values(tpm_values):
		k2r = Databases().k2r

		output_dict = dict()
		annotation_types = set()
		genome_types = set()

		tpm_values_io = open(tpm_values, 'rb')
		tpm_values_io.readline()

		for line in tpm_values_io:
			gene, _, _, _, _, _, _, \
			_, _, _, tpm, \
			_, _,  annotation, sample = line.strip().split(b'\t')
			annotation_list = annotation.split(b',')
			tpm = float(tpm)
			genome = '_'.join(str(gene, "utf-8").split('_')[:2]) # temporary
			genome_types.add(genome)

			if sample not in output_dict:
				output_dict[sample] = dict()
			
			if genome not in output_dict[sample]:
				output_dict[sample][genome] = dict()

			for annotation_type in annotation_list:

				if str(annotation_type, "utf-8") in k2r:
					reactions = k2r[str(annotation_type, "utf-8")]

					for reaction in reactions:
						reaction = str.encode(reaction)

						if reaction not in output_dict[sample][genome]:
							output_dict[sample][genome][reaction] = 0.0
							annotation_types.add(reaction)
					
					output_dict[sample][genome][reaction] += tpm
												
				#else:
				#	
				#	if annotation_type not in output_dict[sample][genome]:
				#		output_dict[sample][genome][annotation_type] = 0.0
				#		annotation_types.add(annotation_type)
				#		
				#	output_dict[sample][genome][annotation_type] += tpm

		return output_dict, annotation_types, genome_types



#!/usr/bin/env python
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################
# Imports
import unittest
import os.path
import sys
import subprocess
import tempfile
import filecmp

###############################################################################

path_to_script 		= os.path.join(os.path.dirname(os.path.realpath(__file__)),'..','bin','enrichm')
path_to_data 		= os.path.join(os.path.dirname(os.path.realpath(__file__)),'data')
path_to_annotate	= os.path.join(path_to_data, 'enrichm_annotate')

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path

###############################################################################

class Tests(unittest.TestCase):

    def test_enrichment_from_ko_matrix(self):

        tmp = tempfile.mkdtemp()
        bin             = os.path.join(path_to_data, 'test_nucleic_bin')
        expected_output = os.path.join(path_to_data, 'enrichm_enrichment_ko')
        metadata        = os.path.join(path_to_data, 'metadata.tsv')
        cmd             = '%s enrichment --annotate_output %s --metadata %s --output %s --force --ko' \
                            % (path_to_script, path_to_annotate, metadata, tmp)
        subprocess.call(cmd, shell=True)
        self.assertTrue(filecmp.dircmp(tmp, expected_output))
        
        for file in os.listdir(tmp):
            if file         == 'enrichment.log': continue
            output_file     = os.path.join(tmp, file)
            expected_file   = os.path.join(expected_output, file)
            self.assertTrue(filecmp.cmp(output_file, expected_file))

    def test_enrichment_from_pfam_matrix(self):
        
        tmp = tempfile.mkdtemp()
        bin             = os.path.join(path_to_data, 'test_nucleic_bin')
        expected_output = os.path.join(path_to_data, 'enrichm_enrichment_pfam')
        metadata        = os.path.join(path_to_data, 'metadata.tsv')
        genomes_to_compare = os.path.join(path_to_data, 'genomes_to_compare.tsv')
        cmd             = '%s enrichment --annotate_output %s --metadata %s --output %s --force --pfam' \
                            % (path_to_script, path_to_annotate, metadata, tmp)

        subprocess.call(cmd, shell=True)
        
        self.assertTrue(filecmp.dircmp(tmp, expected_output))
        for file in os.listdir(tmp):
            if file         == 'enrichment.log': continue
            output_file     = os.path.join(tmp, file)
            expected_file   = os.path.join(expected_output, file)
            self.assertTrue(filecmp.cmp(output_file, expected_file))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__      = "Joel Boyd"
__copyright__   = "Copyright 2017"
__credits__     = ["Joel Boyd"]
__license__     = "GPL3"
__version__     = "0.0.7"
__maintainer__  = "Joel Boyd"
__email__       = "joel.boyd near uq.net.au"
__status__      = "Development"

###############################################################################
# Imports
import os
import urllib
import shutil
import subprocess
import logging
from pathlib import Path
# Local
###############################################################################

class Data:
	'''
	Utilities for archiving, downloading and updating databases.
	'''
	db_var = "ENRICHM_DB"
	if db_var in os.environ:
		DATABASE_DIR 	= os.environ[db_var]
	else:
		DATA_PATH 		= str(Path.home())
		DATABASE_DIR	= os.path.join(DATA_PATH, 'databases')
	VERSION 		= 'VERSION'
	ARCHIVE_SUFFIX 	= '.tar.gz'
	CURRENT_VERSION = open(os.path.join(
		DATABASE_DIR, VERSION)).readline().strip().replace(ARCHIVE_SUFFIX, "")
	
	def __init__(self):
		self.ftp = 'https://data.ace.uq.edu.au/public/enrichm/'

	def _archive_db(self, old_db_file):
		'''
		Archive an old database file
		
		Parameters
		----------
		old_db_file	- String. File name of old database file to archive
		'''
		from enrichm.databases import Databases

		if not os.path.isdir(Databases.OLD_DATABASE_PATH):
			logging.info('Creating directory to store databases: %s' % (Databases.OLD_DATABASE_PATH))
			os.makedirs(Databases.OLD_DATABASE_PATH)
	
		old_db_path_archive \
			= os.path.join(Databases.OLD_DATABASE_PATH, old_db_file + self.ARCHIVE_SUFFIX) 
		old_db_path \
			= os.path.join(self.DATABASE_DIR, old_db_file) 

		logging.info('Compressing old database')
		cmd = "tar -cvzf %s %s > /dev/null" % (old_db_path_archive, old_db_path)
		subprocess.call(cmd, shell=True)

		logging.info('Cleaning up')
		shutil.rmtree(old_db_path)

	def _download_db(self, new_db_file):
		'''
		Download and decompress a new database file
		
		Parameters
		----------
		new_db_file	- String. File name of new database to download and decompress.
		'''
		
		new_db_path_archive \
			= os.path.join(self.DATABASE_DIR, new_db_file)
		logging.info('Downloading new database: %s' % new_db_file)
		cmd = 'wget -q %s -O %s' % (self.ftp + new_db_file, new_db_path_archive)
		subprocess.call(cmd, shell = True)
		cmd = 'wget -q %s -O %s' % (self.ftp + self.VERSION, os.path.join(self.DATABASE_DIR, self.VERSION))
		subprocess.call(cmd, shell = True)
		
		logging.info('Decompressing new database')
		cmd = 'tar -xvzf %s -C %s > /dev/null' % (new_db_path_archive, self.DATABASE_DIR)
		subprocess.call(cmd, shell = True)

		logging.info('Cleaning up')
		os.remove(new_db_path_archive)
	
	def do(self, uninstall):
		'''
		Check database versions, if they're out of date, archive the old and download the new.
		'''

		if uninstall:
			for file in os.listdir(self.DATABASE_DIR):
				file_path = os.path.join(self.DATABASE_DIR, file)
				if os.path.isdir(file_path):
					shutil.rmtree(file_path)
				else:
					os.remove(file_path)
			os.rmdir(self.DATABASE_DIR)

		else:
			version_remote = urllib.request.urlopen(self.ftp + self.VERSION).readline().strip().decode("utf-8")

			if os.path.isdir(self.DATABASE_DIR):
				version_local  = open(os.path.join(self.DATABASE_DIR, self.VERSION)).readline().strip()
				if version_local!=version_remote:
					logging.info('New database found. Archiving old database.')
					self._archive_db(version_local.replace(self.ARCHIVE_SUFFIX,''))
					self._download_db(version_remote)
				else:
					logging.info('Database is up to date!')
			else:
				logging.info('Creating file to store databases.')
				os.makedirs(self.DATABASE_DIR)
				self._download_db(version_remote)
		



#!/usr/bin/env python3
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################


__author__ = "Ben Woodcroft, Joel Boyd"
__copyright__ = "Copyright 2017"
__credits__ = ["Ben Woodcroft"]
__license__ = "GPL3+"
__maintainer__ = "Joel Boyd"
__email__ = "joel.boyd3 near uq.edu.au"
__status__ = "Development"

###############################################################################
# Imports
import re
###############################################################################

KEGG = '^(K\d{5})$'
GH = '(GH\d+)'
PL = '(PL\d+)'
TIGRFAM = '^(TIGR\d{5})$'
PFAM = '(PF\d+)'
CE = '(CE\d+)'
EC = '\\d{1,2}(\\.(\\-|\\d{1,2})){3}'

class ModuleDescription:

    def __init__(self, module_description_string):
        self.module_description_string \
            = module_description_string
        self.parsed_module, self.is_single_step \
            = ModuleDescriptionParser().parse_module_string(module_description_string)

        if isinstance(self.parsed_module, ModuleDescriptionOrRelation):
            new_and = ModuleDescriptionAndRelation()
            new_and.relations = [self.parsed_module]
            self.parsed_module = new_and

    def kos(self):
        '''Return an iterable over the total list of KOs in the module'''
        r_kegg = re.compile(KEGG)
        r_gh = re.compile(GH)
        r_pl = re.compile(PL)
        r_tigrfam = re.compile(TIGRFAM)
        r_pfam = re.compile(PFAM)
        r_ce = re.compile(CE)
        r_ec = re.compile(EC)
        
        if any(r_kegg.findall(self.module_description_string)):
            return r_kegg.findall(self.module_description_string)
        elif any(r_gh.findall(self.module_description_string)):
            return r_gh.findall(self.module_description_string)
        elif any(r_pl.findall(self.module_description_string)):
            return r_pl.findall(self.module_description_string)
        elif any(r_tigrfam.findall(self.module_description_string)):
            return r_tigrfam.findall(self.module_description_string)
        elif any(r_pfam.findall(self.module_description_string)):
            return r_pfam.findall(self.module_description_string)
        elif any(r_ce.findall(self.module_description_string)):
            return r_ce.findall(self.module_description_string)
        elif any(r_ec.findall(self.module_description_string)):
            return r_ec.findall(self.module_description_string)

    def num_steps(self):
        if isinstance(self.parsed_module, ModuleDescriptionAndRelation):
            return len(self.parsed_module.relations)
        else:
            raise Exception("Cannot work with non-AND type modules")

    def num_covered_steps(self, ko_set):
        
        if isinstance(self.parsed_module, ModuleDescriptionAndRelation):
            step_cov = 0
            path_cov = 0 
            reac_cov = 0
            ko_path  = dict()

            for idx, m in enumerate(self.parsed_module.relations):
                step_passed, step_counts, reaction_counts, ko = m.satisfied_with(ko_set, list())
                if step_passed:
                    step_cov+=1
                    path_cov+=step_counts
                    ko_path[idx] = ko
            
                reac_cov+=reaction_counts

            return step_cov, path_cov, reac_cov, ko_path
        
        else:
            raise Exception("Cannot work with non-AND type modules")

class ModuleDescriptionAndRelation:
    
    def satisfied_with(self, set_of_kos, kos):    
        counts          = 0
        step_passed     = False
        reaction_counts = 0
        founds          = list()

        for r in self.relations:
            found, count, reaction_count, ko = r.satisfied_with(set_of_kos, kos)
        
            if found:
                founds.append(1)
                counts += count
        
                for k in ko:
        
                    if k not in kos:
                        kos.append(k)
        
            reaction_counts+=reaction_count
        
        step_passed = len(self.relations) == sum(founds)
        
        return step_passed, counts, reaction_counts, kos

class ModuleDescriptionPlusRelation(ModuleDescriptionAndRelation): pass

class ModuleDescriptionOrRelation:
    def satisfied_with(self, set_of_kos, kos):

        counts          = 0
        step_passed     = False
        reaction_counts = 0

        for r in self.relations:
            found, _, reaction_count, ko = r.satisfied_with(set_of_kos, kos)
            if found:
                step_passed = True
                for k in ko:
                    if k not in kos:
                        kos.append(k)
            else:
                for k in ko:
                    if k in kos:
                        kos.remove(k)
            reaction_counts+=reaction_count
        return step_passed, counts, reaction_counts, kos

class ModuleDescriptionKoEntry:
    def __init__(self, ko):
        self.ko = ko

    def satisfied_with(self, set_of_kos, kos):
        found = self.ko in set_of_kos
        count = (1 if found else 0)
        reaction_count = 1
        return found, count, reaction_count, [self.ko]

class ParserHelper: pass

class ModuleDescriptionParser:
    
    def correct_substrings(self, substring_list):
        fixed_substrings = list()
        for substring in substring_list:
            # Omit optional enzyme (e.g. M00372) 
            # or undefined KO group (e.g. M00079)
            if substring.startswith('-'):
                continue 
            # Remove redundant and definitions
            substring=substring.replace(', ', ',')
            fixed_substrings.append(substring)
        return fixed_substrings
        
    def parse_module_string(self, string):

        frags1 = self.split_on_space(string)
        frags1 = self.correct_substrings(frags1) 
        is_single_step = len(frags1) == 1

        if is_single_step:
            if len(self.split_on_comma(frags1[0]))>1:
                frags1=self.split_on_comma(frags1[0])
                master_relation = ModuleDescriptionOrRelation()
            elif len(self.split_on_plus(frags1[0]))>1:
                frags1=self.split_on_plus(frags1[0])
                master_relation = ModuleDescriptionAndRelation() 
            else:
                frags1=self.split_on_comma(frags1[0])
                master_relation = ModuleDescriptionOrRelation()
        else:
            master_relation = ModuleDescriptionAndRelation()
        
        current = ParserHelper()
        current.top_relation = master_relation
        current.understuff = frags1
        stack = list([current])

        while len(stack) > 0:
            current = stack.pop()
            new_stuff = list()
        
            for e in current.understuff:
        
                if isinstance(e, str):
                    
                    if (re.match(KEGG, e) or 
                        re.match(GH, e) or
                        re.match(PL, e) or
                        re.match(CE, e) or
                        re.match(TIGRFAM, e) or
                        re.match(PFAM, e) or
                        re.match(EC, e)):

                        new_stuff.append(ModuleDescriptionKoEntry(e))

                    else:
                        frags = self.split_on_comma(e)

                        if len(frags) == 1:
                            topush = ParserHelper()
                            comma_splits = self.split_on_space(e)
                            m = None

                            if len(comma_splits) == 1:
                                plus_splits = self.split_on_plus(e)
                                minus_splits = self.split_on_minus(e)

                                if len(plus_splits)>1:
                                    m = ModuleDescriptionPlusRelation()
                                    topush.understuff = plus_splits

                                elif len(minus_splits)>1:
                                    m = ModuleDescriptionAndRelation()
                                    topush.understuff = minus_splits[:1]

                                else:
                                    raise Exception("Parse exception on %s" % string)

                            else:
                                m = ModuleDescriptionAndRelation()
                                topush.understuff = comma_splits

                            topush.top_relation = m
                            stack.append(topush)
                            new_stuff.append(m)

                        else:
                            m = ModuleDescriptionOrRelation()
                            topush = ParserHelper()
                            topush.top_relation = m
                            topush.understuff = frags
                            stack.append(topush)
                            new_stuff.append(m)

                else:
                    new_stuff.append(e)

            current.top_relation.relations = new_stuff

        return master_relation, is_single_step


    def split_on(self, string, characters):
        bracket_counter = 0
        fragments = list()
        current = list()
        remove_end_brackets = True

        for i in range(len(string)):
            c = string[i]
        
            if c == '(':
                current += c
                bracket_counter += 1
        
            elif c == ')':
                current += c
                bracket_counter -= 1
                if bracket_counter == 0 and i < len(string)-1:
                    remove_end_brackets = False
        
            elif c in characters and bracket_counter == 0:
                fragments.append(''.join(current))
                current = ''
        
            else:
                current += c
        
        fragments.append(''.join(current))

        if remove_end_brackets and string[0] == '(':
        
            if string[-1] != ')': raise Exception("Parse error")
            return self.split_on(string[1:-1], characters)

        return fragments

    def split_on_space(self, string):
        return(self.split_on(string, ' '))

    def split_on_plus(self, string):
        return(self.split_on(string, '+'))

    def split_on_comma(self, string):
        return(self.split_on(string, ','))

    def split_on_minus(self, string):
        return(self.split_on(string, '-'))


#!/usr/bin/env python3
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__      = "Joel Boyd"
__copyright__   = "Copyright 2017"
__credits__     = ["Joel Boyd"]
__license__     = "GPL3"
__version__     = "0.0.7"
__maintainer__  = "Joel Boyd"
__email__       = "joel.boyd near uq.net.au"
__status__      = "Development"
 
###############################################################################
# Imports
import logging
# Local
from enrichm.databases import Databases
###############################################################################

class KeggMatrix:
    
    def __init__(self, matrix):
        d = Databases()
        self.r2k = d.r2k
        logging.info("Parsing input matrix: %s" % matrix)
        self.orthology_matrix \
                 = self._parse_matrix(matrix)
        logging.info("Calculating reaction abundances")
        self.reaction_matrix  \
                 = self._calculate_abundances(self.r2k, self.orthology_matrix)
   
    def _calculate_expression_matrix(self, matrix, transcriptome_matrix):
        '''
        '''
        output_dictionary = dict()
        for sample, abundances in transcriptome_matrix.items():
            output_dictionary[sample] = dict()
            for ko, abundance in abundances.items():

                if ko in matrix[sample]:
                    mg_abundance = float(matrix[sample][ko])
                    mt_abundance = float(abundance)
                    if mg_abundance>0:
                        output_dictionary[sample][ko] \
                            = mt_abundance/mg_abundance
                else:
                    continue 
                    # for now ignore genes that are expressed, but not detected
                    # in the metagenome.
        return output_dictionary        
        
    def _parse_matrix(self, matrix):
        
        output_dict = dict()
        
        for idx, line in enumerate(open(matrix)):
            sline = line.strip().split('\t')
            if idx==0:
                self.sample_names = sline[1:]
                    
                for sample in self.sample_names: 
                    output_dict[sample] = dict()
            else:   
                ko_id      = sline[0]
                abundances = sline[1:]
                
                for abundance, sample in zip(abundances, self.sample_names):
                    try:
                        output_dict[sample][ko_id] = float(abundance)
                    except:
                        output_dict[sample][ko_id] = abundance
                        
        return output_dict
    
    def group_abundances(self, samples, reference_dict):
        
        output_dict = dict()
        
        for sample in samples:
            new_dict = {key:entry for key,entry in reference_dict.items() if 
                        key in samples}
            reference_list = list(new_dict[list(new_dict.keys())[0]].keys())

            for reference in reference_list:
                # If samples are missing from stored data, this will crash 
                try:
                    abundances = [new_dict[sample][reference] for sample in samples]
                    average    = sum(abundances)
                    if average > 0:
                        output_dict[reference] = average
               
                except:
                    raise Exception("metadata description does not match input matrix")
        
        return output_dict
                          
    def _calculate_abundances(self, reference_dict, matrix_dict):
        
        output_dict_mean   = dict()

        for sample, ko_abundances in matrix_dict.items():
            output_dict_mean[sample]   = dict()
        
            for reaction, ko_list in reference_dict.items():
                abundances = list()
        
                for ko in ko_list:
        
                    if ko in ko_abundances:

                        if ko_abundances[ko]>0:
                        
                            abundances.append(ko_abundances[ko])
        
                    else:
                        logging.debug("ID not found in input matrix: %s" % ko)
        
                if any(abundances):
                    abundance_mean = sum(abundances)/len(abundances) # average of the abundances...
        
                else:
                    abundance_mean = 0
                
                output_dict_mean[sample][reaction] = abundance_mean
        
        return output_dict_mean



#!/usr/bin/env python
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################
# Imports
import unittest
import os.path
import sys
import subprocess

###############################################################################

path_to_script = os.path.join(os.path.dirname(os.path.realpath(__file__)),'..','bin','enrichm')
path_to_data = os.path.join(os.path.dirname(os.path.realpath(__file__)),'data')
sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path

###############################################################################

class Tests(unittest.TestCase):

    def test_hello_world(self):
        cmd = '%s -h > /dev/null' % path_to_script
        subprocess.call(cmd, shell=True)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__      = "Joel Boyd"
__copyright__   = "Copyright 2018"
__credits__     = ["Joel Boyd"]
__license__     = "GPL3"
__version__     = "0.0.7"
__maintainer__  = "Joel Boyd"
__email__       = "joel.boyd near uq.net.au"
__status__      = "Development"

###############################################################################
# Imports
import logging
import sys
import os
import shutil
import time

# Local
from enrichm.data import Data
from enrichm.network_analyzer import NetworkAnalyser
from enrichm.enrichment import Enrichment
from enrichm.annotate import Annotate
from enrichm.classifier import Classify
from enrichm.generate import GenerateModel
from enrichm.predict import Predict
from enrichm.connect import Connect
###############################################################################

debug={1:logging.CRITICAL,
       2:logging.ERROR,
       3:logging.WARNING,
       4:logging.INFO,
       5:logging.DEBUG}

###############################################################################

class Run:
    
    def __init__(self):
        
        self.ANNOTATE        = 'annotate'
        self.COMPARE         = 'compare'

        self.CLASSIFY        = 'classify'
        self.BUILD           = 'build'
        self.ENRICHMENT      = 'enrichment'
        self.MODULE_AB       = 'module_ab'
        self.DATA            = 'data'
        self.PREDICT         = 'predict'
        self.GENERATE        = 'generate'
        self.CONNECT         = 'connect'

    def _logging_setup(self, args):

        if args.verbosity not in range(1, 6):
            raise Exception("Logging verbosity must be a positive integer between 1 and 5.")

        logger = logging.getLogger('')
        logger.setLevel(debug[args.verbosity])
        log_format = logging.Formatter(fmt="[%(asctime)s] %(levelname)s: %(message)s",
                                       datefmt="%Y-%m-%d %H:%M:%S %p")

        stream_logger = logging.StreamHandler(sys.stdout)
        stream_logger.setFormatter(log_format)
        stream_logger.setLevel(debug[args.verbosity])
        logger.addHandler(stream_logger)

        if args.subparser_name!=self.DATA:
            file_logger = logging.FileHandler(os.path.join(args.output, args.log), 'a')
            file_logger.setFormatter(log_format)
            logger.addHandler(file_logger)

    def _check_general(self, args):
        '''
        Check general input and output options are valid.

        Parameters
        ----------
        args    - object. Argparse object
        '''

        dependencies = {'hmmsearch': "http://hmmer.org/download.html",
                        'diamond': "https://github.com/bbuchfink/diamond",
                        'R': "https://www.r-project.org",
                        'parallel': "https://www.gnu.org/software/parallel",
                        'prodigal': "https://github.com/hyattpd/Prodigal/wiki/installation",
                        'mmseqs': "https://github.com/soedinglab/MMseqs2"}
        
        missing_dependencies = list()
        
        for dependency in dependencies.keys():

            if shutil.which(dependency) == None:
                missing_dependencies.append(dependency)
        
        if len(missing_dependencies)>0:
            dependency_string = '\n'.join(['\t%s\t%s' % (d, dependencies[d]) for d in missing_dependencies])
            raise Exception('The following dependencies need to be installed to run enrichm:\n%s' % (dependency_string))

        # We dont need an output directory for the DATA pipeline
        if args.subparser_name!=self.DATA:
            # Set up working directory
            if not args.output:
                args.output = '%s-enrichm_%s_output' % (time.strftime("%Y-%m-%d_%H-%M"), args.subparser_name)

            if(os.path.isdir(args.output) or os.path.isfile(args.output)):

                if args.force:

                    if os.path.isdir(args.output):
                        shutil.rmtree(args.output)

                    else:
                        os.remove(args.output)

                else:
                    raise Exception("File '%s' exists." % args.output)

            os.mkdir(args.output)   

    def _check_annotate(self, args):
        '''
        Check annotate input and output options are valid.

        Parameters
        ----------
        args    - object. Argparse object
        '''
        if args.cut_ko:
            if int(Data.CURRENT_VERSION.split('_')[-1].replace('v', '')) < 10:
                raise Exception("EnrichM database needs to be version 10 or higher to use KO HMM cutoffs. Please run enrichm data.")

        # ensure either a list of genomes or a directory of genomes have been specified
        if not(args.genome_files or args.genome_directory or args.protein_directory or args.protein_files):
            
            raise Exception("Input error: Either a list of genomes or a directory of genomes need to be specified.")

        if len([x for x in [args.genome_files, args.genome_directory, args.protein_directory, args.protein_files] if x]) != 1:
            
            raise Exception("Input error: Only one type of input can be specified (--genome_files, --genome_directory, --protein_directory, or --protein_files).")
        
        if not args.suffix:
            
            if(args.genome_directory or args.genome_files):
                args.suffix = '.fna'
            
            elif(args.protein_directory or args.protein_files):
                args.suffix = '.faa'
        
        if(args.id>1 or args.id<0):
            
            raise Exception("Identity (--id) must be between 0 and 1.")
        
        if(args.aln_query>1 or args.aln_query<0):
           
            raise Exception("Alignment to query cutoff (--aln_query) must be between 0 and 1")
        
        if(args.aln_reference>1 or args.aln_reference<0):
           
            raise Exception("Alignment to reference cutoff (--aln_reference) must be between 0 and 1")
        
        if any([args.cut_ga, args.cut_nc, args.cut_tc]):
           
            if len([x for x in [args.cut_ga, args.cut_nc, args.cut_tc] if x])>1:
                
                raise Exception("Only one of the following can be selected: --cut_ga, --cut_nc, --cut_tc")
            
            if args.evalue:
                logging.warning('selecting one of the following overrides evalue thresholds: --cut_ga, --cut_nc, --cut_tc')
            
    def _check_enrichment(self, args):
        '''
        Check enrichment input and output options are valid.

        Parameters
        ----------
        args    - object. Argparse object

        Output
        ------
        '''
        ### ~ TODO: Check Multi test correction inputs...
        types = [args.ko, args.pfam, args.tigrfam, args.hypothetical, args.cazy, args.ec, args.ko_hmm]
        
        if not any(types):
            
            raise Exception("Input Error: One of the following flags must be specified: --ko --pfam --tigrfam --hypothetical --cazy")
        
        if len([x for x in types if x])>1:
            
            raise Exception("Only one of the following flags may be specified: --ko --pfam --tigrfam --hypothetical --cazy")

        if not args.abundance and args.abundance_metadata:
           raise Exception("values for both --abundance and --abundance_metadata are required") 

    def _check_classify(self, args):  
        '''
        Check classify input and output options are valid.
        
        Parameters
        ----------
        args    - object. Argparse object
        '''
        # Ensure either an annotation matrix or list file has been specified:
        if not(args.genome_and_annotation_file or args.genome_and_annotation_matrix):
            
            raise Exception("Input error: An input file must be specified to either \
--genome_and_annotation_file or --genome_and_annotation_matrix")

    def _check_build(self, args):
        '''
        Check build input and output options are valid.
        
        Parameters
        ----------
        args    - object. Argparse object
        '''
        if not(args.metadata or args.abundances):
            
            raise Exception("No metadata or abundance information provided to build.")

    def _check_network(self, args):
        '''
        Check network (explore, pathway) input and output options are valid.
        
        Parameters
        ----------
        args    - object. Argparse object
        '''

        if any([args.abundance, args.abundance_metadata]):
            if not (args.abundance and args.abundance_metadata):
                raise Exception("Both abundance and abundance metadata need to be specified")

        if args.subparser_name==NetworkAnalyser.PATHWAY:
            args.depth              = None
            args.queries            = None
            args.starting_compounds = None
            args.steps              = None
            args.number_of_queries  = None
        
        if args.subparser_name==NetworkAnalyser.TRAVERSE:
            args.depth              = None
            args.queries            = None
    
        if args.subparser_name==NetworkAnalyser.EXPLORE:
            args.filter             = None  
            args.limit              = None 
            args.starting_compounds = None
            args.steps              = None
            args.number_of_queries  = None

            if not(args.queries):

                if args.depth:
                    logging.warning("--depth argument ignored without --queries flag")

    def _check_predict(self, args):
        '''
        Inputs
        ------
        
        Outputs
        -------
        '''
        pass

    def _check_generate(self, args):
        '''
        Parameters
        ----------
        
        Output
        ------
        '''
        pass

    def _check_connect(self, args):
        '''
        Check connect
        
        Inputs
        ------
        
        Outputs
        -------
        
        '''
        if(args.cutoff > 1.0 and args.cutoff < 0.0):
           
           raise Exception("Cutoff needs to be between 0 - 1") 
    
    def main(self, args, command):
        '''
        Parameters
        ----------
        
        Output
        ------
        '''
        self._check_general(args)
        self._logging_setup(args)

        logging.info("Running command: %s" % ' '.join(command))

        if args.subparser_name == self.DATA:
            d = Data()
            d.do(args.uninstall)
        
        if args.subparser_name == self.ANNOTATE:
            self._check_annotate(args)
            a = Annotate(# Define inputs and outputs
                         args.output,
                         # Define type of annotation to be carried out
                         args.ko,
                         args.ko_hmm,
                         args.pfam,
                         args.tigrfam,
                         args.hypothetical,
                         args.cazy,
                         args.ec,
                         # Cutoffs
                         args.evalue,
                         args.bit,
                         args.id,
                         args.aln_query, 
                         args.aln_reference, 
                         args.c,
                         args.cut_ga,
                         args.cut_nc,
                         args.cut_tc,
                         args.cut_ko,
                         args.inflation,
                         args.chunk_number,
                         args.chunk_max,
                         args.count_domains,
                         # Parameters
                         args.threads,
                         args.parallel,
                         args.suffix,
                         args.light)
            a.do(args.genome_directory,
                 args.protein_directory, 
                 args.genome_files,
                 args.protein_files)

        elif args.subparser_name == self.CLASSIFY:
            self._check_classify(args)
            c = Classify()
            c.do(args.custom_modules, 
                 args.cutoff,
                 args.genome_and_annotation_file,
                 args.genome_and_annotation_matrix,
                 args.output)

        elif args.subparser_name == self.ENRICHMENT: 
            self._check_enrichment(args)
            e = Enrichment()
            e.do(# Input options
                 args.annotate_output,
                 args.metadata,
                 args.abundance,
                 args.abundance_metadata,
                 # Runtime options
                 args.genomes_to_compare_with_group,
                 args.pval_cutoff,
                 args.proportions_cutoff,
                 args.threshold,
                 args.multi_test_correction,
                 args.batchfile,
                 args.processes,
                 args.ko,
                 args.pfam,
                 args.tigrfam,
                 args.hypothetical,
                 args.cazy,
                 args.ec,
                 args.ko_hmm,
                 # Outputs
                 args.output)

        elif args.subparser_name == self.CONNECT:
            self._check_connect(args)
            c = Connect()
            c.do(args.annotate_output,
                 args.metadata,
                 args.custom_modules,
                 args.cutoff,
                 args.output)

        elif args.subparser_name == NetworkAnalyser.PATHWAY:
            self._check_network(args)
            na=NetworkAnalyser(args.metadata)
            na.do(args.matrix,
                  args.tpm_values,
                  args.abundance,
                  args.abundance_metadata,
                  args.metabolome,
                  args.enrichment_output,
                  args.depth,
                  args.filter,
                  args.limit,
                  args.queries,
                  args.starting_compounds, 
                  args.steps,
                  args.number_of_queries,
                  args.output)
        
        if args.subparser_name == self.PREDICT:
            self._check_predict(args)
            p = Predict()
            p.do(args.forester_model_directory,
                 args.input_matrix,
                 args.output)

        elif args.subparser_name == self.GENERATE:
            self._check_generate(args)
            gm = GenerateModel()
            gm.do(args.input_matrix,
                  args.groups,
                  args.model_type,
                  args.testing_portion,
                  args.grid_search,
                  args.threads,
                  args.output)
        
        logging.info('Done!')
